@@ -27,6 +27,7 @@
 #include <log4cxx/logger.h>
 #include <lsr/ls_fileio.h>
 #include <socket/coresocket.h>
+#include "socket/gsockaddr.h"
 #include <util/pcutil.h>
 #include <util/rlimits.h>
 #include <util/stringtool.h>
@@ -137,8 +138,6 @@ int SUExec::spawnChild(const char *pAppCmd, int fdIn, int fdOut,
         dup2(fdOut, STDOUT_FILENO);
         close(fdOut);
     }
-    else
-        close(STDOUT_FILENO);
 
     if (pLimits)
     {
@@ -219,9 +218,9 @@ int SUExec::checkLScgid(const char *path)
     return 0;
 }
 
-
+#define _HAS_LVE_
 static char sDefaultPath[] = "PATH=/bin:/usr/bin:/usr/local/bin";
-//static char sLVE[] = "LVE_ENABLE=1";
+static char sLVE[] = "LVE_ENABLE=1";
 
 
 int SUExec::suEXEC(const char *pServerRoot, int *pfd, int listenFd,
@@ -430,10 +429,18 @@ int SUExec::cgidSuEXEC(const char *pServerRoot, int *pfd, int listenFd,
 
 
 
-    CoreSocket::connect(
+    int ret = CoreSocket::connect(
         CgidWorker::getCgidWorker()->getConfig().getServerAddr(), 0,
         &fdReq, 1);
 
+    if (ret)
+    {
+        const GSockAddr &server = CgidWorker::getCgidWorker()->
+                                getConfig().getServerAddr();
+        LS_ERROR("[suEXEC] Failed to connect %s, return %d",
+                 server.toString(), ret);
+    }
+    
     if (fdReq != -1)
     {
         m_req.finalize(0, CgidWorker::getCgidWorker()->getConfig().getSecret(),
