@@ -56,6 +56,10 @@ private:
     H2Connection();
 public:
     virtual ~H2Connection();
+
+    LogSession *getLogSession() const
+    {   return getStream();   }
+
     int onReadEx();
     int onReadEx2();
     int onWriteEx();
@@ -75,7 +79,7 @@ public:
             wanted = 2048;
         return wanted;
     }
-
+    
     int flush();
 
     int onCloseEx();
